/*****************************************************************************

 main.cc

 created: 00/11/07  by  Robin Gruber <archi@robinsnest.de>

 Revision: 1  $Author: gruber    $Date: 00/11/07$   

 (C) by Institute of Robotics and Mechatronics, DLR

 Description:

  software tool that locates a sound source
  needs: 2 souncards + ALSA driver
  
        
 Contains:         
 
 Dependencies:     

 Target platform: linux

 *****************************************************************************/

#include <qapplication.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "sndctrl.h"
#include "meter.h"
#include "oszi.h"
#include "minenergy.h"
#include "speechactivity.h"
#include "outputplugin.h"
#include "showmicpos.h"

  
void printusage(void)
{
  fprintf(stderr,"\n\nsoundsource Version 0.9 (c) 2000 Robin Gruber <archi@robinsnest.de>\n\n");
  fprintf(stderr,"usage:\n");
  fprintf(stderr,"soundsource [OPTIONS]\n\n");
  fprintf(stderr,"-m[nsf <name>] operaton mode:\n");
  fprintf(stderr,"   n           operation with soundcards (default)\n");
  fprintf(stderr,"   s           operation with simulated data\n");
  fprintf(stderr,"   f <name>    operation with data from files (name01.raw .. namexx.raw)\n");
  fprintf(stderr,"-r <rate>      samplerate (default: 48000Hz)\n");
  fprintf(stderr,"-o             show oszilloscope\n");
  fprintf(stderr,"-l             show level meter\n");
  fprintf(stderr,"-p             show microphone positions\n");
  fprintf(stderr,"-t             test mode, no beamformer, manual steered simulated microphone\n");
  fprintf(stderr,"-b <rate>      activate simulated microphone with output sample rate <rate>\n");
  fprintf(stderr,"-h <angle>     set horizontal searching angle to +/-<angle> degree (defalut:180)\n");
  fprintf(stderr,"-v <angle>     set vertical searching angle to +/-<angle> degree (default: 0)\n");
  fprintf(stderr,"-c             verbose mode, show graphics for speechactivity, beamformer\n");
  fprintf(stderr,"               and steerd microphone (lots of things blinkin' and movin'!)\n");
  fprintf(stderr,"-a             measurement mode, output simulated microphone directions to\n");
  fprintf(stderr,"               stderr\n");
  fprintf(stderr,"\n");
  fprintf(stderr,"\n");
  exit(1);
}

int main(int argc, char ** argv)
{
  Sound * soundApp;
  Meter * meter;
  Oszi * oszi;
  Minenergy * minenergy=NULL;
  OutputPlugin * outputplugin=NULL;
  ShowMicPos * showmicpos=NULL;
  SpeechActivity * speechactivity=NULL;
  SweepControl * sweepcontrol=NULL;
  unsigned int k;
  int i;
  int mode=MODE_NORMAL;
  int samplerate=48000;
  bool showoszi=false;
  bool showmeter=false;
  bool showpos=false;
  int outputrate=-1;
  int horizontalangle=180;
  int verticalangle=0;
  bool testmode=false;
  bool verbosemode=false;
  bool measmode=false;
  char * inputfilename=NULL;
  // first: check command-line parameters
  if (argc==1) {printusage();}
  for (i=1; i<argc; i++) {
    if (argv[i][0]=='-') {
      for (k=1; k<strlen(argv[i]); k++) {
        switch (argv[i][k]) {
        case 'm': {
          switch (argv[i][k+1]) {
          case 'n': {mode=MODE_NORMAL; k++; break;}
          case 's': {mode=MODE_SIMULATION; k++; break;}
          case 'f': {
            mode=MODE_FILEIO; 
            i++;  
            k=100000; 
            if (i<argc) {inputfilename=argv[i];}
            else {fprintf(stderr,"Missing argument to -mf!\n"); exit(1);}
            break;}
          default: {fprintf(stderr,"\nIllegal option -m%c!\n",argv[i][k+1]); exit(1); break;}
          }
          break;
        case 'r':{
          i++; k=100000;
          if (i<argc) {samplerate=strtol(argv[i],NULL,10);}
          else {fprintf(stderr,"Missing argument to -r!\n"); exit(1);}
          ; break;}
        case 'o':{showoszi=true; break;}
        case 'l':{showmeter=true; break;}
        case 'p':{showpos=true; break;}
        case 'b':{
          i++; k=100000;
          if (i<argc) {outputrate=strtol(argv[i],NULL,10); break;}
          else {fprintf(stderr,"Missing argument to -b!\n");exit(1);}
        }
        case 'h':{
          i++; k=100000;
          if (i<argc) {
            horizontalangle=strtol(argv[i],NULL,10);
            fprintf(stderr,"Horizontal angle is +/-%d\n",horizontalangle);
          }
          else {fprintf(stderr,"Missing parameter to -h!\n"); exit(1); }
          break;}
        case 'v':{
          i++; k=100000;
          if (i<argc) {
            verticalangle=strtol(argv[i],NULL,10);
            fprintf(stderr,"Vertical angle is +/-%d\n",verticalangle);
          }
          else { fprintf(stderr,"Missing argument to -v!\n"); exit(1);}
          break;}
        }
        case 't':{testmode=true; break;}
        case 'c':{verbosemode=true; break;}
        case 'a':{measmode=true; break;}
        default: {fprintf(stderr,"Unknown parameter: %c\n",argv[i][k]); exit(1); }
        }
      }
    }
  }
  if ((samplerate>100000)||(samplerate<8000)) {
     fprintf(stderr,"Illegal samplerate specified!\n"); 
     exit(1); 
  }
  if ((outputrate>0)&&(outputrate<1000)) {
     fprintf(stderr,"Warning: Strange outputsamplerate specified!\n");
  }
  if (outputrate>samplerate) {
     fprintf(stderr,"Upsamplig of output not possible! Use a higher input samplerate!\n");
     exit(1);
  }
  QApplication a(argc, argv);
  soundApp = new Sound(4,samplerate,mode,inputfilename); 
  if (showmeter) {meter = new Meter(soundApp,NULL);}
  if (showoszi) {oszi = new Oszi(soundApp,NULL);}
  if (!testmode){
     minenergy = new Minenergy(horizontalangle, verticalangle, verbosemode, soundApp,NULL);
  }
  if (outputrate!=-1) {
     // means: we want a signal output, so generate a outputplugin
     outputplugin = new OutputPlugin(measmode, outputrate, verbosemode, soundApp);
     if (testmode) {
        // testmode? manual sweepcontrol, no voiceactivity detector!
        sweepcontrol = new SweepControl();
        QObject::connect(sweepcontrol,SIGNAL(sweepTo(double,double)),
                outputplugin,SLOT(sweepTo(double,double)));
     }
     else {
        // normal mode? we need a voiceactivity detector
        speechactivity = new SpeechActivity(verbosemode, soundApp,NULL);
        QObject::connect(speechactivity,SIGNAL(speechActivityChanged(bool)),
                         outputplugin,SLOT(speechActivityChanged(bool)));
        QObject::connect(minenergy,SIGNAL(sweepTo(double,double)),
                         outputplugin,SLOT(sweepTo(double,double)));
     }
  }
  if (showpos) {showmicpos = new ShowMicPos();}
  soundApp->startSampling();
  return a.exec();
} 






/*****************************************************************************
 $Log: $
 *****************************************************************************/
