// Meter application
#include <qpainter.h>
#include <qpixmap.h>
#include <stdlib.h>
#include <math.h>
#include "sndctrl.h"
#include "debug.h"
#include "tools.h"
#include "meter.h"


#define DEBUG(x) 

Meter::Meter(Sound * soundApp, QWidget * parent=NULL, const char * name=NULL) 
    : QFrame(parent, "Meter")
{
    int i;
    last=(long long*)malloc(soundApp->getChannels()*sizeof(long long));
    for (i=0; i<soundApp->getChannels(); i++) {
	last[i]=0;
    }
    receiver=new SoundReceiver(soundApp,soundApp->getChannels());
    receiver->setFrameSize(READSIZE);
    receiver->setStepSize(READSIZE);
    connect(receiver,SIGNAL(newDataReady(int,int, signed short **)),
    	    this,SLOT(newDataReady(int, int, signed short **)));
    // allocate the buffer for the meter bars
    pm = new QPixmap(5+25*soundApp->getChannels(),220);
    setFixedSize(5+25*soundApp->getChannels(),220);
    setGeometry(5,85,width(),height());
    setCaption("Meter");
    show();
    pm->fill();
    repaint();
}

Meter::~Meter()
{
    delete receiver;
    free(last);
}

void Meter::newDataReady(int channels, int frameSize, signed short ** data)
{
    int i,k;
    long long power;
    char c[2];
    signed short * d;
    QPainter p;
    DEBUG(fprintf(stderr,"Meter: data received!\n"));
    pm->fill();
    p.begin(pm);
    for (k=0; k<channels; k++) {
	power=0;
        // sum energy over one frame
        d=data[k];
	for (i=0; i<frameSize; i++) {
	    power+=(sqr(*(d++)));
	}
        // logaritmate the energy
	power=int(double(log((power>>10)/frameSize))*7);
        // slowly decrease the old value
	last[k]=last[k]*9/10;
        // if the old value is larger than the actual one use it instead
        // this gives a smoother bar
	if (power>last[k]) {last[k]=power;}
	else {power=last[k];}
	for (i=0; (i<90)&&(i<power); i++) {
            switch(i) {
            case 0: 
                p.setPen(QColor(120,255,255,QColor::Hsv));// green
                break;
            case 69: 
                p.setPen(QColor(60,255,255,QColor::Hsv)); // yellow
                break;
            case 84: 
                p.setPen(QColor(0,255,255,QColor::Hsv)); // red
                break;
            }
          p.drawLine(k*25+5,190-2*i,k*25+25,190-2*i);
	}
    }
    // description
    for (k=0; k<channels; k++) {
	p.setPen(QColor(360/channels*k,255,255,QColor::Hsv));
	c[0]='A'+k;
	c[1]=0;
	p.drawText(10+k*25,210,c);
    }
    p.end();
    repaint(FALSE);
}

void Meter::paintEvent(QPaintEvent *)
{
    bitBlt(this,0,0,pm,0,0,pm->width(),pm->height(),CopyROP,TRUE);
}
