// Speech activity detection application
// This program is part of the Diplomarbeit of Robin Gruber (c) 2000
// see README for details

#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include "speechactivity.h"
#include "sndctrl.h"
#include "tools.h"
#include "debug.h"

#define DEBUG(x) 

#define FRAMESIZE 4096
#define INDICATOR 20
#define MAXTHRESH 40000000
#define DELAY 1
#define THRESH 40 
#define LOWPASS 40

SpeechActivity::SpeechActivity(bool verbosemode, Sound * soundApp, QWidget * parent=NULL, const char * name=NULL) 
    : QFrame(parent, "Meter")
{
   int indwidth=soundApp->getChannels()*25+5;
   receiver=new SoundReceiver(soundApp,soundApp->getChannels());
   receiver->setFrameSize(FRAMESIZE);
   receiver->setStepSize(FRAMESIZE/2);
   connect(receiver,SIGNAL(newDataReady(int,int, signed short **)),
           this,SLOT(newDataReady(int, int, signed short **)));
   verbose=verbosemode;
   if (verbose) {
       pmred = new QPixmap(indwidth,INDICATOR);
       pmgreen = new QPixmap(indwidth,INDICATOR);
       setFixedSize(indwidth,INDICATOR);
       setGeometry(5,335,width(),height());
       pmred->fill(red);
       pmgreen->fill(green);
       show();
       setCaption("Speech activity");
   }
    threshhold=MAXTHRESH/2;
    delay=0;
}

SpeechActivity::~SpeechActivity()
{
    delete receiver;
    free(result);
}

void SpeechActivity::newDataReady(int channels, int frameSize, signed short ** data)
{
    int i,k;
    unsigned int p,l;
    p=0;
    // calculate the power of the sampled signal
    for (i=0; i<frameSize; i++) {
        l=0;
        for (k=0; k<channels; k++) {
            l+=data[k][i];
        }
        p+=(l*l)/frameSize;
    }
    // if power is THRESH above the background noise-level
    // speech seems to be present!
    if (p>threshhold+threshhold*THRESH/100+THRESH*10000) {
        active=true; 
        delay=DELAY;
        emit(speechActivityChanged(active));
    } 
    // if power is below background noise-level 
    // and timer is zero, speech has finished
    if (p<=threshhold*105/100) {
        if (delay==0) {
            active=false;
            emit(speechActivityChanged(active));
        }
        else {delay--;}
    }
    // threshhold is the low-pass filtered value of p so we know the 
    // background noise level
    threshhold=(threshhold*(LOWPASS-1)+p)/LOWPASS;
    if (threshhold>MAXTHRESH) {threshhold=MAXTHRESH;}
    DEBUG(fprintf(stderr,"SpeechActivity: threshhold: %10d\n",threshhold));
    if (verbose) {repaint(FALSE);}
}

void SpeechActivity::paintEvent(QPaintEvent *)
{
  // show green indicator if active, red if no speech detected
    if (!active) {
        bitBlt(this,0,0,pmred,0,0,pmred->width(),pmred->height(),CopyROP,TRUE);
    } else {
        bitBlt(this,0,0,pmgreen,0,0,pmgreen->width(),pmgreen->height(),CopyROP,TRUE);
    }
}
