/*
**************************************
*                                    *
* visualize sound                    *
* 2000-11-17 Robin Gruber            *
*                                    *
**************************************
*/

#include "soundvision.h"

#define BLOCKLENGTH 512
#define WIDTH 200
#define SAMPLERATE 8000

void hammingwindow(int length, signed short * a, signed short * b,
		   signed short * c, signed short * d, double * frame)
{
    int i;
    double factor;
    for (i=0; i<length/4; i++){	    
	factor=0.54-0.46*cos(2*M_PI*double(i)/double(length));
	frame[i]=double(a[i])*factor;
	frame[length-i-1]=double(d[length/4-i-1])*factor;
	factor=0.54-0.46*cos(2*M_PI*double(i+length/4)/double(length));
	frame[i+length/4]=double(b[i])*factor;
	frame[length-i-1-length/4]=double(c[length/4-i-1])*factor;
    }
}


EnergyFFT::EnergyFFT(int frameSize)
{
  framesize=frameSize;
  printf("initializing FFT!\n");
  int i;
  double phi;
  si=(double*)malloc(framesize*sizeof(double));
  co=(double*)malloc(framesize*sizeof(double));
  phi=M_PI/double(framesize/2);
  for (i=0; i<framesize/2; i++) {
    si[i]=-sin(double(i)*phi);
    co[i]=cos(double(i)*phi);
  }    
  iframe=(double*) malloc(framesize*sizeof(double));
}


EnergyFFT::~EnergyFFT()
{
  free(iframe);
  free(si);
  free(co);
}

void EnergyFFT::calc(double * rframe)
{
    int n2;
    int i,j,k,m,mm;
    double fact;
    double phi;
    double temp1, temp2, rew, imw;
    n2=framesize/2;
    phi=M_PI/double(n2);
    fact=1.0/double(framesize);
    for (i=0; i<framesize; i++) {
	rframe[i]*=fact;
	iframe[i]=0.0;
    }
    // Bitumkehr
    j=0;
    for (i=1; i<framesize; i++) {
	m=n2;
	while (j>=m) {
	    j-=m;
	    m/=2;
	}
	j+=m;
	if (j>i) {
	    temp1=rframe[i]; rframe[i]=rframe[j]; rframe[j]=temp1;
	    temp2=iframe[i]; iframe[i]=iframe[j]; iframe[j]=temp2;
	}
    }
    // ln(n) Ueberlagerungen
    for (mm=2; mm<=framesize; mm*=2) 
      { 
	m=mm/2;
	for (k=0; k<m; k++) {
	    rew=co[k*n2];
	    imw=si[k*n2];
	    for (i=k; i<framesize; i+=mm) {
		j=i+m;
		temp1=rew*rframe[j]-imw*iframe[j];
		temp2=imw*rframe[j]+rew*iframe[j];
		rframe[j]=rframe[i]-temp1;
		iframe[j]=iframe[i]-temp2;
		rframe[i]+=temp1;
		iframe[i]+=temp2;
	    }
	}
	n2/=2;
    }
    for (i=0; i<framesize/2; i++) {
	rframe[i]=sqrt(sqrt(rframe[i]*rframe[i]+iframe[i]*iframe[i]))*12;
    }
}


SoundVision::SoundVision(int blockLength, QWidget * parent=NULL, char * name=NULL) 
  : QFrame(parent, name)
{
  blocklength=blockLength;
  blockmarker=0;
  linecount=0;
  inblock[0]=(signed short *)malloc(blocklength/4*sizeof(short));
  inblock[1]=(signed short *)malloc(blocklength/4*sizeof(short));
  inblock[2]=(signed short *)malloc(blocklength/4*sizeof(short));
  inblock[3]=(signed short *)malloc(blocklength/4*sizeof(short));
  frame=(double*)malloc(blocklength*sizeof(double));
  average=(double*)malloc(blocklength/2*sizeof(double));
  if ((inblock[0]==NULL)||(inblock[1]==NULL)||
      (inblock[2]==NULL)||(inblock[3]==NULL)||
      (frame==NULL)||(average==NULL)) {
    fprintf(stderr,"Cannot get enogh memory!\n");
    exit(-2);
  }
  for (i=0; i<blocklength/2; i++) {average[i]=0;}
  fft=new EnergyFFT(blocklength);
  if ((id=openDSP("/dev/dsp",RECORD,16,SAMPLERATE,MONO))<0) {
    printf("Error opening DSP!\n"); 
    exit(-1);
  }
  setCaption("soundvision    scale: 1000ms / 500Hz");
  pm = new QPixmap(WIDTH, blocklength/2);
  pm->fill();
  setMinimumWidth(time);
  setFixedHeight(blocklength/2);
  setPalette( QPalette( QColor(255, 255, 255) ) );
  startTimer((1000*blocklength/SAMPLERATE/4/10)*10);
  dist=500*BLOCKLENGTH/SAMPLERATE;
  time=SAMPLERATE*4/BLOCKLENGTH;
  show();
}


SoundVision::~SoundVision()
{
  free(inblock[0]);
  free(inblock[1]);
  free(inblock[2]);
  free(inblock[3]);
  free(frame);
  free(average);
  delete pm;
  closeDSP(id);
}


void SoundVision::timerEvent(QTimerEvent *)
{
  //  printf("timer event!\n");
  QPainter p;
  if (record(id, (char*)inblock[blockmarker],blocklength/2)<0) {
    fprintf(stderr,"Cannot read from device!\n");
    exit(-3);
  }
  hammingwindow(blocklength,inblock[(blockmarker+1)%4],
                inblock[(blockmarker+2)%4],inblock[(blockmarker+3)%4],
                inblock[blockmarker],frame);
  blockmarker=(blockmarker+1)%4;
  fft->calc(frame);
  if (linecount>=width()) {linecount=0;}
  p.begin(pm);
  for (i=0; i<blocklength/2; i++) {
    average[i]=average[i]*0.999+0.001*frame[i];
    frame[i]-=average[i];
    if (frame[i]<0) {frame[i]=0;}
    if (frame[i]>255) {frame[i]=255;}
    p.setPen(QColor(255-int(frame[i]),255-int(frame[i]),255-int(frame[i]),QColor::Rgb));
    p.drawPoint(linecount,blocklength/2-i);
    // draws the matrix
    if (((i % dist==0)&&(linecount%5==0))||((linecount%time==0)&&(i%5==0))) {
       p.setPen(blue);
       p.drawPoint(linecount,blocklength/2-i);
    }
  }
  p.end();
  bitBlt(this,linecount,0,pm,linecount,0,1,pm->height(),CopyROP,TRUE);
  linecount++;
}


void SoundVision::paintEvent(QPaintEvent *)
{
   int i;
   // draw whole buffer to screen
  bitBlt(this,0,0,pm,0,0,pm->width(),pm->height(),CopyROP,TRUE);
}

void SoundVision::resizeEvent(QResizeEvent *)
{
  delete pm;
  pm = new QPixmap(width(), blocklength/2);
  pm->fill();
  linecount=0;
}

int main(int argc, char * argv[])
{
  SoundVision * soundvision;
  QApplication a(argc, argv);
  soundvision = new SoundVision(BLOCKLENGTH);
  a.setMainWidget(soundvision);
  return a.exec();
}

